@@ -1,4 +1,3 @@
-# coding=utf-8
 # Copyright (c) 2019 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,26 +15,9 @@
 import logging
 import sys
 
-import absl.logging
+from ludwig.globals import LUDWIG_VERSION as __version__  # noqa
 
-from ludwig.globals import LUDWIG_VERSION as __version__
+logging.basicConfig(level=logging.INFO, stream=sys.stdout, format="%(message)s")
 
-# Tensorflow 1.14 has compatibility issues with python native logging
-# This was one of the suggested solutions to continue using native logging
-# https://github.com/tensorflow/tensorflow/issues/26691
-logging.root.removeHandler(absl.logging._absl_handler)
-absl.logging._warn_preinit_stderr = False
-
-logger = logging.getLogger(__name__)
-# Default logging level for the project
-logger.setLevel(logging.INFO)
-
-# Configure stream handler
-ch = logging.StreamHandler(sys.stdout)
-ch.setLevel(logging.DEBUG)
-
-# Set formatter
-formatter = logging.Formatter('%(message)s')
-ch.setFormatter(formatter)
-
-logger.addHandler(ch)
+# Disable annoying message about NUMEXPR_MAX_THREADS
+logging.getLogger("numexpr").setLevel(logging.WARNING)
