@@ -18,15 +18,51 @@ from typing import Dict, List, Optional, Union
 
 import torch
 
+from ludwig.api_annotations import DeveloperAPI
 from ludwig.constants import CATEGORY
 from ludwig.encoders.base import Encoder
 from ludwig.encoders.registry import register_encoder
 from ludwig.modules.embedding_modules import Embed
-from ludwig.schema.encoders.category_encoders import CategoricalEmbedConfig, CategoricalSparseConfig
+from ludwig.schema.encoders.category_encoders import (
+    CategoricalEmbedConfig,
+    CategoricalPassthroughEncoderConfig,
+    CategoricalSparseConfig,
+)
 
 logger = logging.getLogger(__name__)
 
 
+@DeveloperAPI
+@register_encoder("passthrough", [CATEGORY])
+class CategoricalPassthroughEncoder(Encoder):
+    def __init__(self, input_size=1, encoder_config=None, **kwargs):
+        super().__init__()
+        self.config = encoder_config
+
+        logger.debug(f" {self.name}")
+        self.input_size = input_size
+
+    def forward(self, inputs, mask=None):
+        """
+        :param inputs: The inputs fed into the encoder.
+               Shape: [batch x 1]
+        """
+        return inputs.float()
+
+    @staticmethod
+    def get_schema_cls():
+        return CategoricalPassthroughEncoderConfig
+
+    @property
+    def input_shape(self) -> torch.Size:
+        return torch.Size([self.input_size])
+
+    @property
+    def output_shape(self) -> torch.Size:
+        return self.input_shape
+
+
+@DeveloperAPI
 @register_encoder("dense", CATEGORY)
 class CategoricalEmbedEncoder(Encoder):
     def __init__(
@@ -82,6 +118,7 @@ class CategoricalEmbedEncoder(Encoder):
         return torch.Size([1])
 
 
+@DeveloperAPI
 @register_encoder("sparse", CATEGORY)
 class CategoricalSparseEncoder(Encoder):
     def __init__(
