@@ -15,28 +15,17 @@
 # ==============================================================================
 import logging
 import os
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import Dict, List, Optional, Tuple, Union
 
 import numpy as np
 import torch
 import torchaudio
 
-from ludwig.constants import (
-    AUDIO,
-    AUDIO_FEATURE_KEYS,
-    COLUMN,
-    ENCODER,
-    NAME,
-    PREPROCESSING,
-    PROC_COLUMN,
-    SRC,
-    TIED,
-    TYPE,
-)
+from ludwig.constants import AUDIO, AUDIO_FEATURE_KEYS, COLUMN, NAME, PREPROCESSING, PROC_COLUMN, SRC, TYPE
 from ludwig.features.base_feature import BaseFeatureMixin
 from ludwig.features.sequence_feature import SequenceInputFeature
 from ludwig.schema.features.audio_feature import AudioInputFeatureConfig
-from ludwig.schema.features.utils import register_input_feature
+from ludwig.types import FeatureMetadataDict, PreprocessingConfigDict, TrainingSetMetadataDict
 from ludwig.utils.audio_utils import (
     calculate_mean,
     calculate_var,
@@ -53,17 +42,16 @@ from ludwig.utils.audio_utils import (
 )
 from ludwig.utils.data_utils import get_abs_path
 from ludwig.utils.fs_utils import has_remote_protocol
-from ludwig.utils.misc_utils import set_default_value, set_default_values
+from ludwig.utils.misc_utils import set_default_value
 from ludwig.utils.types import TorchscriptPreprocessingInput
 
 logger = logging.getLogger(__name__)
 
 
 class _AudioPreprocessing(torch.nn.Module):
-
     audio_feature_dict: Dict[str, Union[float, int, str]]
 
-    def __init__(self, metadata: Dict[str, Any]):
+    def __init__(self, metadata: TrainingSetMetadataDict):
         super().__init__()
         self.audio_feature_dict = {
             key: value
@@ -99,17 +87,15 @@ class AudioFeatureMixin(BaseFeatureMixin):
     def type():
         return AUDIO
 
-    @staticmethod
-    def preprocessing_defaults():
-        return AudioInputFeatureConfig().preprocessing.to_dict()
-
     @staticmethod
     def cast_column(column, backend):
         return column
 
     @staticmethod
-    def get_feature_meta(column, preprocessing_parameters, backend):
-        first_audio_file_path = column.head(1)[0]
+    def get_feature_meta(
+        column, preprocessing_parameters: PreprocessingConfigDict, backend, is_input_feature: bool
+    ) -> FeatureMetadataDict:
+        first_audio_file_path = column.head(1).iloc[0]
         _, sampling_rate_in_hz = torchaudio.load(first_audio_file_path)
 
         feature_dim = AudioFeatureMixin._get_feature_dim(preprocessing_parameters, sampling_rate_in_hz)
@@ -125,7 +111,7 @@ class AudioFeatureMixin(BaseFeatureMixin):
         }
 
     @staticmethod
-    def _get_feature_dim(preprocessing_parameters, sampling_rate_in_hz):
+    def _get_feature_dim(preprocessing_parameters: PreprocessingConfigDict, sampling_rate_in_hz):
         feature_type = preprocessing_parameters[TYPE]
 
         if feature_type == "raw":
@@ -349,7 +335,13 @@ class AudioFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def add_feature_data(
-        feature_config, input_df, proc_df, metadata, preprocessing_parameters, backend, skip_save_processed_input
+        feature_config,
+        input_df,
+        proc_df,
+        metadata,
+        preprocessing_parameters: PreprocessingConfigDict,
+        backend,
+        skip_save_processed_input,
     ):
         set_default_value(feature_config["preprocessing"], "in_memory", preprocessing_parameters["in_memory"])
 
@@ -405,13 +397,13 @@ class AudioFeatureMixin(BaseFeatureMixin):
                 backend,
             )
             proc_df[feature_config[PROC_COLUMN]] = audio_features
-        else:
-            backend.check_lazy_load_supported(feature_config)
 
         return proc_df
 
     @staticmethod
-    def _get_max_length_feature(preprocessing_parameters, sampling_rate_in_hz, audio_length_limit_in_s):
+    def _get_max_length_feature(
+        preprocessing_parameters: PreprocessingConfigDict, sampling_rate_in_hz, audio_length_limit_in_s
+    ):
         feature_type = preprocessing_parameters[TYPE]
         audio_length_limit_in_samp = audio_length_limit_in_s * sampling_rate_in_hz
 
@@ -435,10 +427,8 @@ class AudioFeatureMixin(BaseFeatureMixin):
             raise ValueError(f"{feature_type} is not recognized.")
 
 
-@register_input_feature(AUDIO)
 class AudioInputFeature(AudioFeatureMixin, SequenceInputFeature):
-    def __init__(self, input_feature_config: Union[AudioInputFeatureConfig, Dict], encoder_obj=None, **kwargs):
-        input_feature_config = self.load_config(input_feature_config)
+    def __init__(self, input_feature_config: AudioInputFeatureConfig, encoder_obj=None, **kwargs):
         super().__init__(input_feature_config, encoder_obj=encoder_obj, **kwargs)
 
         if not getattr(self.encoder_obj.config, "embedding_size", None):
@@ -464,20 +454,13 @@ class AudioInputFeature(AudioFeatureMixin, SequenceInputFeature):
         return torch.float32
 
     @staticmethod
-    def update_config_with_metadata(input_feature, feature_metadata, *args, **kwargs):
-        input_feature[ENCODER]["max_sequence_length"] = feature_metadata["max_length"]
-        input_feature[ENCODER]["embedding_size"] = feature_metadata["feature_dim"]
-        input_feature[ENCODER]["should_embed"] = False
+    def update_config_with_metadata(feature_config, feature_metadata, *args, **kwargs):
+        feature_config.encoder.max_sequence_length = feature_metadata["max_length"]
+        feature_config.encoder.embedding_size = feature_metadata["feature_dim"]
+        feature_config.encoder.should_embed = False
 
     @staticmethod
-    def populate_defaults(input_feature):
-        defaults = AudioInputFeatureConfig()
-        set_default_values(
-            input_feature, {ENCODER: {TYPE: defaults.encoder.type}, PREPROCESSING: {}, TIED: defaults.tied}
-        )
-
-    @staticmethod
-    def create_preproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
+    def create_preproc_module(metadata: TrainingSetMetadataDict) -> torch.nn.Module:
         return _AudioPreprocessing(metadata)
 
     @staticmethod
