@@ -25,7 +25,7 @@ from ludwig.globals import LUDWIG_VERSION
 from ludwig.hyperopt.run import hyperopt
 from ludwig.utils.data_utils import load_config_from_str, load_yaml
 from ludwig.utils.defaults import default_random_seed
-from ludwig.utils.print_utils import logging_level_registry, print_ludwig
+from ludwig.utils.print_utils import get_logging_level_registry, print_ludwig
 
 logger = logging.getLogger(__name__)
 
@@ -141,7 +141,7 @@ def hyperopt_cli(
         for training.
     :param gpu_memory_limit: (float: default: `None`) maximum memory fraction
         [0, 1] allowed to allocate per GPU device.
-    :param allow_parallel_threads: (bool, default: `True`) allow TensorFlow
+    :param allow_parallel_threads: (bool, default: `True`) allow PyTorch
         to use multithreading parallelism to improve performance at
         the cost of determinism.
     :param callbacks: (list, default: `None`) a list of
@@ -401,7 +401,7 @@ def cli(sys_argv):
     for callback in args.callbacks:
         callback.on_cmdline("hyperopt", *sys_argv)
 
-    args.logging_level = logging_level_registry[args.logging_level]
+    args.logging_level = get_logging_level_registry()[args.logging_level]
     logging.getLogger("ludwig").setLevel(args.logging_level)
     global logger
     logger = logging.getLogger("ludwig.hyperopt")
