@@ -15,21 +15,28 @@
 import logging
 from abc import ABC, abstractmethod
 from contextlib import contextmanager
-from typing import Any, Callable, Generator, Optional
+from typing import Any, Callable, Generator, Optional, Type
 
 import torch
-import torchmetrics.functional as metrics_F
 from torch import Tensor, tensor
-from torchmetrics import Accuracy as _Accuracy
-from torchmetrics import MeanAbsoluteError
+from torchmetrics import AUROC, CharErrorRate, MeanAbsoluteError
 from torchmetrics import MeanMetric as _MeanMetric
 from torchmetrics import MeanSquaredError, Metric
+from torchmetrics.classification import (
+    BinaryAccuracy,
+    BinaryPrecision,
+    BinaryRecall,
+    BinarySpecificity,
+    MulticlassAccuracy,
+)
 from torchmetrics.functional.regression.r2 import _r2_score_compute, _r2_score_update
 from torchmetrics.metric import jit_distributed_available
+from torchmetrics.text.perplexity import Perplexity
 
 from ludwig.constants import (
     ACCURACY,
     BINARY,
+    BINARY_WEIGHTED_CROSS_ENTROPY,
     CATEGORY,
     HITS_AT_K,
     JACCARD,
@@ -40,26 +47,31 @@ from ludwig.constants import (
     MEAN_SQUARED_ERROR,
     MINIMIZE,
     NUMBER,
+    PERPLEXITY,
+    PRECISION,
     PREDICTIONS,
     PROBABILITIES,
     R2,
+    RECALL,
     ROC_AUC,
     ROOT_MEAN_SQUARED_ERROR,
     ROOT_MEAN_SQUARED_PERCENTAGE_ERROR,
     SEQUENCE,
+    SEQUENCE_ACCURACY,
     SET,
+    SPECIFICITY,
     TEXT,
     TOKEN_ACCURACY,
     VECTOR,
 )
+from ludwig.distributed import get_current_dist_strategy
 from ludwig.modules.loss_modules import (
     BWCEWLoss,
     SequenceSoftmaxCrossEntropyLoss,
     SigmoidCrossEntropyLoss,
     SoftmaxCrossEntropyLoss,
 )
-from ludwig.modules.metric_registry import metric_registry, register_metric
-from ludwig.utils.horovod_utils import gather_all_tensors, is_distributed_available
+from ludwig.modules.metric_registry import get_metric_objective, get_metric_registry, register_metric
 from ludwig.utils.loss_utils import rmspe_loss
 from ludwig.utils.metric_utils import masked_correct_predictions
 from ludwig.utils.torch_utils import sequence_length_2D
@@ -67,25 +79,15 @@ from ludwig.utils.torch_utils import sequence_length_2D
 logger = logging.getLogger(__name__)
 
 
+def _gather_all_tensors_fn() -> Optional[Callable]:
+    get_current_dist_strategy().gather_all_tensors_fn()
+
+
 class LudwigMetric(Metric, ABC):
     @classmethod
     def can_report(cls, feature: "OutputFeature") -> bool:  # noqa: F821
         return True
 
-    @classmethod
-    @abstractmethod
-    def get_objective(cls):
-        raise NotImplementedError()
-
-    @classmethod
-    @abstractmethod
-    def get_inputs(cls):
-        """Returns the key of the tensor from the predictions() Dict that should be used for computing this metric.
-
-        For example: PREDICTIONS would be used for accuracy metrics while LOGITS would be used for loss metrics.
-        """
-        raise NotImplementedError()
-
     @contextmanager
     def sync_context(
         self,
@@ -95,12 +97,13 @@ class LudwigMetric(Metric, ABC):
         should_unsync: bool = True,
         distributed_available: Optional[Callable] = jit_distributed_available,
     ) -> Generator:
-        """Override the behavior of this in the base class to support Horovod."""
+        """Override the behavior of this in the base class to support custom distributed strategies."""
+        dist_strategy = get_current_dist_strategy()
         self.sync(
-            dist_sync_fn=gather_all_tensors,
+            dist_sync_fn=dist_strategy.gather_all_tensors_fn(),
             process_group=process_group,
             should_sync=should_sync,
-            distributed_available=is_distributed_available,
+            distributed_available=dist_strategy.is_available,
         )
 
         yield
@@ -108,106 +111,62 @@ class LudwigMetric(Metric, ABC):
         self.unsync(should_unsync=self._is_synced and should_unsync)
 
 
-@register_metric(ROOT_MEAN_SQUARED_ERROR, [NUMBER])
+@register_metric(ROOT_MEAN_SQUARED_ERROR, [NUMBER], MINIMIZE, PREDICTIONS)
 class RMSEMetric(MeanSquaredError, LudwigMetric):
     """Root mean squared error metric."""
 
     def __init__(self, **kwargs):
-        super().__init__(squared=False, dist_sync_fn=gather_all_tensors, **kwargs)
-
-    @classmethod
-    def get_objective(cls):
-        return MINIMIZE
-
-    @classmethod
-    def get_inputs(cls):
-        return PREDICTIONS
+        super().__init__(squared=False, dist_sync_fn=_gather_all_tensors_fn(), **kwargs)
 
 
-@register_metric(ROC_AUC, [BINARY])
-class ROCAUCMetric(LudwigMetric):
-    """Fast implementation of metric for area under ROC curve."""
+@register_metric(PRECISION, [BINARY], MAXIMIZE, PROBABILITIES)
+class PrecisionMetric(BinaryPrecision, LudwigMetric):
+    """Precision metric."""
 
-    def __init__(
-        self,
-        num_thresholds: int = 201,
-        epsilon: float = 1e-7,
-        **kwargs,
-    ) -> None:
-        super().__init__(dist_sync_fn=gather_all_tensors)
-        self.num_thresholds = num_thresholds
-        self.epsilon = epsilon
-        self.add_state("summary_stats", torch.zeros(num_thresholds, 4), dist_reduce_fx="sum")
+    def __init__(self, **kwargs):
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
 
-    def _get_thresholds(self, device, dtype) -> Tensor:
-        thresholds = torch.linspace(0, 1, self.num_thresholds, device=device, dtype=dtype)
-        thresholds[0] -= self.epsilon
-        thresholds[-1] += self.epsilon
-        return thresholds
+
+@register_metric(RECALL, [BINARY], MAXIMIZE, PROBABILITIES)
+class RecallMetric(BinaryRecall, LudwigMetric):
+    """Recall metric."""
+
+    def __init__(self, **kwargs):
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
+
+
+@register_metric(ROC_AUC, [BINARY], MAXIMIZE, PROBABILITIES)
+class BinaryAUROCMetric(AUROC, LudwigMetric):
+    """Area under the receiver operating curve."""
+
+    def __init__(self, **kwargs):
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
 
     def update(self, preds: Tensor, target: Tensor) -> None:
-        # Currently only supported for binary tasks.
-        if preds.ndim > 1 or target.ndim > 1:
-            raise RuntimeError(
-                f"Only binary tasks supported, but received input of "
-                f"{max(preds.ndim, target.ndim)} dimensions while expecting"
-                f"1-dimensional input."
-            )
+        super().update(preds, target.type(torch.int8))
 
-        if torch.min(preds) < 0 or torch.max(preds) > 1:
-            raise RuntimeError(
-                f"Only binary tasks supported, but received predictions in range "
-                f"({torch.min(preds)}, {torch.max(preds)})."
-            )
 
-        thresholds = self._get_thresholds(preds.device, preds.dtype)
-        target = target.to(bool).type(preds.dtype)
+@register_metric(ROC_AUC, [CATEGORY], MAXIMIZE, PROBABILITIES)
+class CategoryAUROCMetric(AUROC, LudwigMetric):
+    """Area under the receiver operating curve."""
 
-        preds = preds.unsqueeze(1)
-        target = target.unsqueeze(1)
+    def __init__(self, num_classes: int, **kwargs):
+        super().__init__(num_classes=num_classes, dist_sync_fn=_gather_all_tensors_fn())
 
-        # Compute correct predictions at each threshold.
-        correct_predictions = ((preds >= thresholds) == target).to(int)
 
-        # Compute true positives, false positives, true negatives, false negatives.
-        # overall_predictions is a tensor where each cell represents the type of prediction:
-        # 0: false positive
-        # 1: true negative
-        # 2: false negative
-        # 3: true positive
-        overall_predictions = correct_predictions + (2 * target)
+@register_metric(SPECIFICITY, [BINARY], MAXIMIZE, PROBABILITIES)
+class SpecificityMetric(BinarySpecificity, LudwigMetric):
+    """Specificity metric."""
 
-        # Sum up the number of true positives, false positives, true negatives, false negatives at each threshold.
-        self.summary_stats += torch.eye(4, device=preds.device)[overall_predictions.T.long()].sum(dim=1, keepdim=False)
-
-    def compute(self) -> Tensor:
-        # Compute true positives, false positives, true negatives, false negatives.
-        self.summary_stats = self.summary_stats.squeeze()
-        false_positives = self.summary_stats[:, 0]
-        true_negatives = self.summary_stats[:, 1]
-        false_negatives = self.summary_stats[:, 2]
-        true_positives = self.summary_stats[:, 3]
-
-        true_positive_rate = true_positives / (true_positives + false_negatives)
-        false_positive_rate = false_positives / (false_positives + true_negatives)
-
-        # Compute area under ROC curve. Multiply by -1 because tpr and fpr are computed from the opposite direction.
-        return -1 * torch.trapz(true_positive_rate, false_positive_rate)
-
-    @classmethod
-    def get_objective(cls):
-        return MAXIMIZE
-
-    @classmethod
-    def get_inputs(cls):
-        return PROBABILITIES
+    def __init__(self, **kwargs):
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
 
 
 class MeanMetric(LudwigMetric):
     """Abstract class for computing mean of metrics."""
 
     def __init__(self, **kwargs):
-        super().__init__(dist_sync_fn=gather_all_tensors)
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
         self.avg = _MeanMetric()
 
     def update(self, preds: Tensor, target: Tensor) -> None:
@@ -225,32 +184,24 @@ class MeanMetric(LudwigMetric):
         raise NotImplementedError()
 
 
-@register_metric(ROOT_MEAN_SQUARED_PERCENTAGE_ERROR, [NUMBER])
+@register_metric(ROOT_MEAN_SQUARED_PERCENTAGE_ERROR, [NUMBER], MINIMIZE, PREDICTIONS)
 class RMSPEMetric(MeanMetric):
     def __init__(self, **kwargs):
-        super().__init__(dist_sync_fn=gather_all_tensors)
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
 
     """ Root mean squared percentage error metric. """
 
     def get_current_value(self, preds: Tensor, target: Tensor) -> Tensor:
         return rmspe_loss(target, preds)
 
-    @classmethod
-    def get_objective(cls):
-        return MINIMIZE
 
-    @classmethod
-    def get_inputs(cls):
-        return PREDICTIONS
-
-
-@register_metric(R2, [NUMBER, VECTOR])
+@register_metric(R2, [NUMBER, VECTOR], MAXIMIZE, PREDICTIONS)
 class R2Score(LudwigMetric):
     """Custom R-squared metric implementation that modifies torchmetrics R-squared implementation to return Nan
     when there is only sample. This is because R-squared is only defined for two or more samples.
 
-    Custom implementation uses code from torchmetrics v0.9.2's implementation of R2:
-    https://github.com/Lightning-AI/metrics/blob/master/src/torchmetrics/regression/r2.py
+    Custom implementation uses code from torchmetrics v0.9.2's implementation of R2: https://github.com/Lightning-
+    AI/metrics/blob/master/src/torchmetrics/regression/r2.py
     """
 
     def __init__(
@@ -304,16 +255,8 @@ class R2Score(LudwigMetric):
             self.sum_squared_error, self.sum_error, self.residual, self.total, self.adjusted, self.multioutput
         )
 
-    @classmethod
-    def get_objective(cls):
-        return MAXIMIZE
 
-    @classmethod
-    def get_inputs(cls):
-        return PREDICTIONS
-
-
-@register_metric(LOSS, [])
+@register_metric(LOSS, [], MINIMIZE, LOGITS)
 class LossMetric(MeanMetric, ABC):
     def __init__(self):
         super().__init__()
@@ -322,20 +265,12 @@ class LossMetric(MeanMetric, ABC):
     def get_current_value(self, preds: Tensor, target: Tensor) -> Tensor:
         raise NotImplementedError()
 
-    @classmethod
-    def get_objective(cls):
-        return MINIMIZE
-
-    @classmethod
-    def get_inputs(cls):
-        return LOGITS
-
     @classmethod
     def can_report(cls, feature: "OutputFeature") -> bool:  # noqa: F821
         return False
 
 
-@register_metric("binary_weighted_cross_entropy", [BINARY])
+@register_metric(BINARY_WEIGHTED_CROSS_ENTROPY, [BINARY], MINIMIZE, LOGITS)
 class BWCEWLMetric(LossMetric):
     """Binary Weighted Cross Entropy Weighted Logits Score Metric."""
 
@@ -358,7 +293,7 @@ class BWCEWLMetric(LossMetric):
         return self.loss_function(preds, target)
 
 
-@register_metric("softmax_cross_entropy", [CATEGORY])
+@register_metric("softmax_cross_entropy", [CATEGORY], MINIMIZE, LOGITS)
 class SoftmaxCrossEntropyMetric(LossMetric):
     def __init__(self, **kwargs):
         super().__init__()
@@ -368,7 +303,7 @@ class SoftmaxCrossEntropyMetric(LossMetric):
         return self.softmax_cross_entropy_function(preds, target)
 
 
-@register_metric("sequence_softmax_cross_entropy", [SEQUENCE, TEXT])
+@register_metric("sequence_softmax_cross_entropy", [SEQUENCE, TEXT], MINIMIZE, LOGITS)
 class SequenceSoftmaxCrossEntropyMetric(LossMetric):
     def __init__(self, **kwargs):
         super().__init__()
@@ -378,7 +313,7 @@ class SequenceSoftmaxCrossEntropyMetric(LossMetric):
         return self.sequence_softmax_cross_entropy_function(preds, target)
 
 
-@register_metric("sigmoid_cross_entropy", [SET])
+@register_metric("sigmoid_cross_entropy", [SET], MINIMIZE, LOGITS)
 class SigmoidCrossEntropyMetric(LossMetric):
     def __init__(self, **kwargs):
         super().__init__()
@@ -388,10 +323,10 @@ class SigmoidCrossEntropyMetric(LossMetric):
         return self.sigmoid_cross_entropy_function(preds, target)
 
 
-@register_metric(TOKEN_ACCURACY, [SEQUENCE, TEXT])
+@register_metric(TOKEN_ACCURACY, [SEQUENCE, TEXT], MAXIMIZE, PREDICTIONS)
 class TokenAccuracyMetric(MeanMetric):
     def __init__(self, **kwargs):
-        super().__init__(dist_sync_fn=gather_all_tensors)
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
 
     def get_current_value(self, preds: Tensor, target: Tensor) -> Tensor:
         target = target.type(preds.dtype)
@@ -399,111 +334,84 @@ class TokenAccuracyMetric(MeanMetric):
         masked_correct_preds = masked_correct_predictions(target, preds, target_sequence_length)
         return torch.mean(masked_correct_preds)
 
-    @classmethod
-    def get_objective(cls):
-        return MAXIMIZE
 
-    @classmethod
-    def get_inputs(cls):
-        return PREDICTIONS
+@register_metric(SEQUENCE_ACCURACY, [SEQUENCE, TEXT], MAXIMIZE, PREDICTIONS)
+class SequenceAccuracyMetric(MeanMetric):
+    def __init__(self, **kwargs):
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
+
+    def get_current_value(self, preds: Tensor, target: Tensor) -> Tensor:
+        return torch.sum(torch.all(preds == target, dim=1)) / target.size()[0]
 
 
-@register_metric(ACCURACY, [BINARY])
-class Accuracy(_Accuracy, LudwigMetric):
+@register_metric(PERPLEXITY, [SEQUENCE, TEXT], MINIMIZE, PROBABILITIES)
+class PerplexityMetric(Perplexity, LudwigMetric):
+    def __init__(self, **kwargs):
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
+
+    def update(self, preds: Tensor, target: Tensor) -> None:
+        super().update(preds, target.type(torch.int64))
+
+
+@register_metric("char_error_rate", [SEQUENCE, TEXT], MINIMIZE, PREDICTIONS)
+class CharErrorRateMetric(CharErrorRate, LudwigMetric):
+    def __init__(self, **kwargs):
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
+
+
+@register_metric(ACCURACY, [BINARY], MAXIMIZE, PREDICTIONS)
+class Accuracy(BinaryAccuracy, LudwigMetric):
     """R-squared metric."""
 
     def __init__(self, **kwargs):
-        super().__init__(dist_sync_fn=gather_all_tensors)
-
-    @classmethod
-    def get_objective(cls):
-        return MAXIMIZE
-
-    @classmethod
-    def get_inputs(cls):
-        return PREDICTIONS
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
 
 
-@register_metric(ACCURACY, [CATEGORY])
-class CategoryAccuracy(_Accuracy, LudwigMetric):
-    def __init__(self, **kwargs):
-        super().__init__(dist_sync_fn=gather_all_tensors)
+@register_metric(ACCURACY, [CATEGORY], MAXIMIZE, PREDICTIONS)
+class CategoryAccuracy(MulticlassAccuracy, LudwigMetric):
+    def __init__(self, num_classes: int, **kwargs):
+        super().__init__(num_classes=num_classes, dist_sync_fn=_gather_all_tensors_fn())
 
     def update(self, preds: Tensor, target: Tensor) -> None:
-        # make sure y_true is tf.int64
         super().update(preds, target.type(torch.long))
 
-    @classmethod
-    def get_objective(cls):
-        return MAXIMIZE
 
-    @classmethod
-    def get_inputs(cls):
-        # TODO: double check
-        return PREDICTIONS
-
-
-@register_metric(HITS_AT_K, [CATEGORY])
-class HitsAtKMetric(_Accuracy, LudwigMetric):
-    def __init__(self, top_k: int = 3, **kwargs):
-        super().__init__(top_k=top_k, dist_sync_fn=gather_all_tensors)
+@register_metric(HITS_AT_K, [CATEGORY], MAXIMIZE, LOGITS)
+class HitsAtKMetric(MulticlassAccuracy, LudwigMetric):
+    def __init__(self, num_classes: int, top_k: int, **kwargs):
+        super().__init__(num_classes=num_classes, top_k=top_k, dist_sync_fn=_gather_all_tensors_fn(), **kwargs)
 
     def update(self, preds: Tensor, target: Tensor) -> None:
-        super().update(preds, target)
-
-    @classmethod
-    def get_objective(cls):
-        return MAXIMIZE
-
-    @classmethod
-    def get_inputs(cls):
-        return LOGITS
+        super().update(preds, target.type(torch.long))
 
     @classmethod
     def can_report(cls, feature: "OutputFeature") -> bool:  # noqa: F821
         return feature.num_classes > feature.top_k
 
 
-@register_metric(MEAN_ABSOLUTE_ERROR, [NUMBER, VECTOR])
+@register_metric(MEAN_ABSOLUTE_ERROR, [NUMBER, VECTOR], MINIMIZE, PREDICTIONS)
 class MAEMetric(MeanAbsoluteError, LudwigMetric):
     def __init__(self, **kwargs):
-        super().__init__(dist_sync_fn=gather_all_tensors)
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
 
     def update(self, preds: Tensor, target: Tensor) -> None:
         super().update(preds.detach(), target)
 
-    @classmethod
-    def get_objective(cls):
-        return MINIMIZE
 
-    @classmethod
-    def get_inputs(cls):
-        return PREDICTIONS
-
-
-@register_metric(MEAN_SQUARED_ERROR, [NUMBER, VECTOR])
+@register_metric(MEAN_SQUARED_ERROR, [NUMBER, VECTOR], MINIMIZE, PREDICTIONS)
 class MSEMetric(MeanSquaredError, LudwigMetric):
     def __init__(self, **kwargs):
-        super().__init__(dist_sync_fn=gather_all_tensors)
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
 
     def update(self, preds: Tensor, target: Tensor) -> None:
         super().update(preds, target)
 
-    @classmethod
-    def get_objective(cls):
-        return MINIMIZE
 
-    @classmethod
-    def get_inputs(cls):
-        return PREDICTIONS
-
-
-@register_metric(JACCARD, [SET])
+@register_metric(JACCARD, [SET], MAXIMIZE, PROBABILITIES)
 class JaccardMetric(MeanMetric):
     def __init__(self, threshold: float = 0.5, **kwargs):
-        super().__init__(dist_sync_fn=gather_all_tensors)
+        super().__init__(dist_sync_fn=_gather_all_tensors_fn())
         self.threshold = threshold
-        self.add_state(name="loss", default=[], dist_reduce_fx="mean")
 
     def get_current_value(self, preds: Tensor, target: Tensor) -> Tensor:
         # notation: b is batch size and nc is number of unique elements in the set
@@ -517,71 +425,27 @@ class JaccardMetric(MeanMetric):
 
         return intersection / union  # shape [b]
 
-    @classmethod
-    def get_objective(cls):
-        return MAXIMIZE
 
-    @classmethod
-    def get_inputs(cls):
-        return PROBABILITIES
+def get_metric_cls(metric_name: str) -> Type[LudwigMetric]:
+    return get_metric_registry()[metric_name]
 
 
-def get_improved_fun(metric: str) -> Callable:
-    if metric_registry[metric].get_objective() == MINIMIZE:
+def get_improved_fn(metric: str) -> Callable:
+    if get_metric_objective(metric) == MINIMIZE:
         return lambda x, y: x < y
     else:
         return lambda x, y: x > y
 
 
 def get_initial_validation_value(metric: str) -> float:
-    if metric_registry[metric].get_objective() == MINIMIZE:
+    if get_metric_objective(metric) == MINIMIZE:
         return float("inf")
     else:
         return float("-inf")
 
 
 def get_best_function(metric: str) -> Callable:
-    if metric_registry[metric].get_objective() == MINIMIZE:
+    if get_metric_objective(metric) == MINIMIZE:
         return min
     else:
         return max
-
-
-def accuracy(preds: Tensor, target: Tensor) -> Tensor:
-    """
-    Returns:
-        Accuracy (float tensor of shape (1,)).
-    """
-    return metrics_F.accuracy(preds, target)
-    # correct_predictions = predictions == target
-    # accuracy = torch.mean(correct_predictions.type(torch.float32))
-    # return accuracy, correct_predictions
-
-
-def perplexity(cross_entropy_loss):
-    # This seem weird but is correct:
-    # we are returning the cross entropy loss as it will be later summed,
-    # divided by the size of the dataset and finally exponentiated,
-    # because perplexity has a avg_exp aggregation strategy
-    # in the output config in SequenceOutputFeature.
-    # This implies that in Model update_output_stats_batch()
-    # the values read from the perplexity node will be summed
-    # and in Model update_output_stats() they will be divided
-    # by the set size first and exponentiated.
-    return cross_entropy_loss
-
-
-def error(preds: Tensor, target: Tensor) -> Tensor:
-    return target - preds
-
-
-def absolute_error(preds: Tensor, target: Tensor) -> Tensor:
-    return torch.abs(target - preds)
-
-
-def squared_error(preds: Tensor, target: Tensor) -> Tensor:
-    return (target - preds) ** 2
-
-
-def r2(preds: Tensor, target: Tensor) -> Tensor:
-    return metrics_F.r2_score(preds, target)
