@@ -1,8 +1,8 @@
-from typing import ClassVar, Optional
-
+from ludwig.api_annotations import DeveloperAPI
 from ludwig.schema import utils as schema_utils
 
 
+@DeveloperAPI
 class BasePreprocessingConfig(schema_utils.BaseMarshmallowConfig):
     """Base class for input feature preprocessing. Not meant to be used directly.
 
@@ -10,6 +10,3 @@ class BasePreprocessingConfig(schema_utils.BaseMarshmallowConfig):
     from the corresponding input feature class are copied over: check each class to check which attributes are different
     from the preprocessing of each feature.
     """
-
-    feature_type: ClassVar[Optional[str]] = None
-    "Class variable pointing to the corresponding preprocessor."
