@@ -3,8 +3,6 @@ import os
 import subprocess
 import sys
 
-import pytest
-
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.INFO)
 logging.getLogger("ludwig").setLevel(logging.INFO)
@@ -12,7 +10,6 @@ logging.getLogger("ludwig").setLevel(logging.INFO)
 TEST_SCRIPT = os.path.join(os.path.dirname(__file__), "scripts", "run_train_comet.py")
 
 
-@pytest.mark.distributed
 def test_contrib_experiment(csv_filename):
     cmdline = [sys.executable, TEST_SCRIPT, "--csv-filename", csv_filename]
     exit_code = subprocess.call(" ".join(cmdline), shell=True, env=os.environ.copy())
